package me.hammerle.snuviengine.api;

import java.nio.ByteBuffer;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL15.*;
import static org.lwjgl.opengl.GL20.*;
import static org.lwjgl.opengl.GL30.*;

public final class ColorRenderer {
    private final static int BUFFER_BYTE_LENGTH = 1024 * 1024; // 1 MiB
    private final static int OBJECT_LENGTH = 64; // 16 Floats

    private final int vertexArray;
    private final int vertexBuffer;
    private ByteBuffer buffer;
    private int offset = BUFFER_BYTE_LENGTH;
    private float depth = 0.0f;

    protected ColorRenderer() {
        vertexArray = glGenVertexArrays();
        vertexBuffer = glGenBuffers();
        GLHelper.glBindVertexArray(vertexArray);
        GLHelper.glBindBuffer(vertexBuffer);
        glEnableVertexAttribArray(0);
        glEnableVertexAttribArray(2);
        glVertexAttribPointer(0, 3, GL_FLOAT, false, 16, 0);
        glVertexAttribPointer(2, 4, GL_UNSIGNED_BYTE, true, 16, 12);
    }

    public void setDepth(float depth) {
        this.depth = depth;
    }

    public void drawRectangle(float minX, float minY, float maxX, float maxY, int color) {
        buffer = getNextBuffer();
        if(buffer == null) {
            return;
        }
        addToBuffer(minX, maxY, color);
        addToBuffer(minX, minY, color);
        addToBuffer(maxX, maxY, color);
        addToBuffer(maxX, minY, color);
        finishBuffer();
        drawBuffer();
    }

    private ByteBuffer getNextBuffer() {
        GLHelper.glBindBuffer(vertexBuffer);
        offset += OBJECT_LENGTH;
        if(offset + OBJECT_LENGTH >= BUFFER_BYTE_LENGTH) {
            offset = 0;
            glBufferData(GL_ARRAY_BUFFER, BUFFER_BYTE_LENGTH, GL_STREAM_DRAW);
        }
        return glMapBufferRange(GL_ARRAY_BUFFER, offset, OBJECT_LENGTH, GL_MAP_WRITE_BIT | GL_MAP_UNSYNCHRONIZED_BIT, buffer);
    }

    private void addToBuffer(float x, float y, int color) {
        buffer.putFloat(x);
        buffer.putFloat(y);
        buffer.putFloat(depth);
        buffer.putInt(color);
    }

    private void finishBuffer() {
        buffer.flip();
        glUnmapBuffer(GL_ARRAY_BUFFER);
    }

    private void drawBuffer() {
        GLHelper.glBindVertexArray(vertexArray);
        glDrawArrays(GL_TRIANGLE_STRIP, offset / 16, 4);
    }
}
